(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: upositive_rational.ml,v 1.3 2014/01/10 15:28:04 fclement Exp $ *)

(* Positive rational numbers. *)

open Upositive_rational_types;;

type t = Upositive_rational_types.t

let of_nat = to_t;;

let _1_ = of_nat Unatural._1_ Unatural._1_;;

let scale { numerator = p; denominator = q; } n =
  Unatural.div (Unatural.mul n p) q
;;

(* Comparison. *)

let compare
    { numerator = p1; denominator = q1; }
    { numerator = p2; denominator = q2; } =
  Pervasives.compare (Unatural.mul p1 q2) (Unatural.mul p2 q1)
;;

let comparison_lift cmp r1 r2 = cmp (compare r1 r2) 0;;

let equal = comparison_lift ( = );;
let different = comparison_lift ( <> );;
let smaller = comparison_lift ( < );;
let greater = comparison_lift ( > );;
let smaller_or_equal = comparison_lift ( <= );;
let greater_or_equal = comparison_lift ( >= );;

let min r1 r2 = if smaller_or_equal r1 r2 then r1 else r2;;
let max r1 r2 = if greater_or_equal r1 r2 then r1 else r2;;
