(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uset_types.ml,v 1.1 2013/06/04 17:11:41 fclement Exp $ *)

(* The private type of set. *)

let uset_types_invalid_arg msg =
  Pervasives.invalid_arg (Printf.sprintf "Uset.of_array: %s" msg)
;;

type 'a set =
  | Empty of 'a Uuniverse.t
  | Nonempty of 'a nonempty

and 'a nonempty =
  {
   support : 'a Uuniverse.t;
   indices : 'a Uindex.t array;
  }
;;

let compare x y =
  let cmp = Pervasives.compare x y in
  if cmp = 0 then uset_types_invalid_arg "duplication detected" else
  cmp
;;

let to_nonempty u indices =
  if indices = [||] then uset_types_invalid_arg "empty array" else
  let len = Array.length indices in
  Array.sort compare indices;
  Uuniverse.check_index "of_array" u indices.(pred len);
  { support = u; indices = indices; }
;;
